@@ -1,21 +1,26 @@
-# Copyright 1999-2009 Gentoo Foundation
+# Copyright 1999-2018 Gentoo Foundation
 # Distributed under the terms of the GNU General Public License v2
-# $Id: CompositeTask.py 13993 2009-08-11 07:47:27Z zmedico $
 
 from _emerge.AsynchronousTask import AsynchronousTask
 from portage import os
 
 class CompositeTask(AsynchronousTask):
 
-	__slots__ = ("scheduler",) + ("_current_task",)
+	__slots__ = ("_current_task",)
+
+	_TASK_QUEUED = -1
 
 	def isAlive(self):
 		return self._current_task is not None
 
-	def cancel(self):
-		self.cancelled = True
+	def _cancel(self):
 		if self._current_task is not None:
-			self._current_task.cancel()
+			if self._current_task is self._TASK_QUEUED:
+				self.returncode = 1
+				self._current_task = None
+				self._async_wait()
+			else:
+				self._current_task.cancel()
 
 	def _poll(self):
 		"""
@@ -32,7 +37,9 @@ class CompositeTask(AsynchronousTask):
 		prev = None
 		while True:
 			task = self._current_task
-			if task is None or task is prev:
+			if task is None or \
+				task is self._TASK_QUEUED or \
+				task is prev:
 				# don't poll the same task more than once
 				break
 			task.poll()
@@ -40,25 +47,6 @@ class CompositeTask(AsynchronousTask):
 
 		return self.returncode
 
-	def _wait(self):
-
-		prev = None
-		while True:
-			task = self._current_task
-			if task is None:
-				# don't wait for the same task more than once
-				break
-			if task is prev:
-				# Before the task.wait() method returned, an exit
-				# listener should have set self._current_task to either
-				# a different task or None. Something is wrong.
-				raise AssertionError("self._current_task has not " + \
-					"changed since calling wait", self, task)
-			task.wait()
-			prev = task
-
-		return self.returncode
-
 	def _assert_current(self, task):
 		"""
 		Raises an AssertionError if the given task is not the
@@ -76,7 +64,7 @@ class CompositeTask(AsynchronousTask):
 		Subclasses can use this as a generic task exit callback.
 
 		@rtype: int
-		@returns: The task.returncode attribute.
+		@return: The task.returncode attribute.
 		"""
 		self._assert_current(task)
 		if task.returncode != os.EX_OK:
@@ -114,7 +102,21 @@ class CompositeTask(AsynchronousTask):
 		a task.
 
 		"""
+		try:
+			task.scheduler = self.scheduler
+		except AttributeError:
+			pass
 		task.addExitListener(exit_handler)
 		self._current_task = task
 		task.start()
 
+	def _task_queued(self, task):
+		task.addStartListener(self._task_queued_start_handler)
+		self._current_task = self._TASK_QUEUED
+
+	def _task_queued_start_handler(self, task):
+		self._current_task = task
+
+	def _task_queued_wait(self):
+		return self._current_task is not self._TASK_QUEUED or \
+			self.cancelled or self.returncode is not None
