@@ -13,6 +13,11 @@ import signal
 import sys
 import traceback
 
+try:
+	import asyncio as _real_asyncio
+except ImportError:
+	_real_asyncio = None
+
 try:
 	import fcntl
 except ImportError:
@@ -192,8 +197,11 @@ class EventLoop(object):
 			self.IO_OUT = PollConstants.POLLOUT
 			self.IO_PRI = PollConstants.POLLPRI
 
-		self._EVENT_READ = self.IO_IN | self.IO_HUP
-		self._EVENT_WRITE = self.IO_OUT
+		# These trigger both reader and writer callbacks.
+		EVENT_SHARED = self.IO_HUP | self.IO_ERR | self.IO_NVAL
+
+		self._EVENT_READ = self.IO_IN | EVENT_SHARED
+		self._EVENT_WRITE = self.IO_OUT | EVENT_SHARED
 
 		self._child_handlers = {}
 		self._sigchld_read = None
@@ -812,7 +820,14 @@ class EventLoop(object):
 		@raise: the Future's exception
 		"""
 		future = asyncio.ensure_future(future, loop=self._asyncio_wrapper)
-		while not future.done():
+
+		# Since done callbacks are executed via call_soon, it's desirable
+		# to continue iterating until those callbacks have executed, which
+		# is easily achieved by registering a done callback and waiting for
+		# it to execute.
+		waiter = self.create_future()
+		future.add_done_callback(waiter.set_result)
+		while not waiter.done():
 			self.iteration()
 
 		return future.result()
@@ -927,7 +942,11 @@ class EventLoop(object):
 			if executor is None:
 				executor = ForkExecutor(loop=self)
 				self._default_executor = executor
-		return executor.submit(func, *args)
+		future = executor.submit(func, *args)
+		if _real_asyncio is not None:
+			future = _real_asyncio.wrap_future(future,
+				loop=self._asyncio_wrapper)
+		return future
 
 	def is_running(self):
 		"""Return whether the event loop is currently running."""
